#!/usr/bin/env python
'''
    This is the program used for reading the SL_contrib_template and
    spitting out an updateinfo stanza
'''

import os
import sys

from updateinfo.updateinfohelper import UpdateinfoHelper

try:
    from yaml import load
    try:
        from yaml import CLoader as Loader
    except ImportError:
        from yaml import Loader
except ImportError:
    print >> sys.stderr, "Could not import yaml lib"
    print >> sys.stderr, "Try installing PyYAML"
    raise ImportError('', "Cannot import yaml")

class MakeUpdateinfo(UpdateinfoHelper):
    '''
        This is the class doing all the work
    '''
    def __init__(self, configfile, verbose=False):
        UpdateinfoHelper.__init__(self, configfile)

        self.yaml_dir = self.config['datastore']['yamldir']

        self.build(verbose=verbose)

        self.unknown_packages.sort()
        for i in self.unknown_packages:
            print i

    def updateid_from_package(self, package):
        '''
            Given a package name, what is its update id?
        '''
        found = False
        package = os.path.basename(package)
        for filename in os.listdir(self.yaml_dir):
            if not filename.endswith('.yaml'):
                continue

            _fd = open(self.yaml_dir + '/' + filename, 'r')
            data = load(_fd, Loader=Loader)
            _fd.close()
            for rpm in data['rpms']:
                for entry in rpm:
                    if package in rpm[entry]:
                        found = True

            if found:
                pkg_id = None
                for rpm in data['rpms']:
                    if rpm.has_key('SL6_SRPMS'):
                        pkg_id = rpm['SL6_SRPMS'][0]
                if pkg_id != None:
                    pkg_id = pkg_id.replace('.src.rpm','')
                    return (pkg_id, pkg_id)

    def add_entry(self, tuv_updateid, my_updateid):
        '''
            What do we know about this update id.
            We don't care about collections, packages, or references here
             just stuff that pertains to this update

            Once you've got the data here, simply add the entry in

            Descriptions of these values can be found in the
             updateinfo.xsd
        '''
        _fd = open(self.yaml_dir + '/' + my_updateid +'.yaml')
        data = load(_fd, Loader=Loader)
        _fd.close()

        update_title =  data['package']
        update_issue_date = data['release_date']
        update_description = data['release_description']
        update_description = update_description.rstrip().lstrip()

        for person in data['submitted_by']:
            if person['primary_contact']:
                update_from = person['email']

        update_status = self.config['status']
        if not data['production_ready']:
            update_status = 'prerelease'

        update_type = 'enhancement'
        if data['security_update'] != False:
            update_type = 'security'
            update_severity = data['security_update']
        else:
            update_severity = None
            if data.has_key('update_type'):
                update_type = data['update_type']

        update_modifcation_date = None
        update_summary = None
        update_rights = None
        update_solution = None

        update_reboot = False
        update_restart = False
        update_relogin = False

        if 'desktop must be restarted' in update_description:
            update_relogin = True
        elif 'must be restarted' in update_description:
            update_restart = True
        if 'system must be rebooted' in update_description:
            update_reboot = True
        elif 'system rebooted' in update_description:
            update_reboot = True

        self._add_entry(my_updateid, update_from, update_status, update_type,
                       update_title, update_issue_date, update_modifcation_date,
                       update_description, update_summary, update_rights,
                       update_solution, update_severity, update_reboot,
                       update_restart, update_relogin)

    def add_references(self, tuv_updateid, my_updateid):
        '''
            What references are attached to this update?
             Find them and add them in
        '''
        _fd = open(self.yaml_dir + '/' + my_updateid +'.yaml')
        data = load(_fd, Loader=Loader)
        _fd.close()

        if data['fixed_bugs'] not in (False, None, 'False', 'None'):
            for bug in data['fixed_bugs']:
                self._add_references(my_updateid, bug['type'], bug['url'],
                                     bug['id'], bug['summary'])
        if data['fixed_cves'] not in (False, None, 'False', 'None'):
            for bug in data['fixed_bugs']:
                self._add_references(my_updateid, 'cve',
                               self.config['ref_hints']['cve_base'] + bug['id'],
                               bug['id'], bug['summary'])

        if data['release_announcement'] not in (False, None, 'False', 'None'):
            self._add_references(my_updateid, 'self',
                                 data['release_announcement'], my_updateid,
                                 'Release Announcement')

        # Low, Moderate, Important, Critical?
        if data['security_update'] not in (False, None, 'False', 'None'):
            sev_url = self.config['ref_hints']['cve_base']['severity_ref']
            self._add_references(my_updateid, 'other',
                                 sev_url + data['security_update'],
                                 data['security_update'],
                                 'Issue Severity Classification')

if __name__ == "__main__":
    import optparse
    OPTPARSER = optparse.OptionParser()
    OPTPARSER.add_option('--configfile', help='''What File''')
    OPTPARSER.add_option('--verbose', action='store_true', help='''Be chatty''')
    OPTIONS = OPTPARSER.parse_args()[0]

    if OPTIONS.configfile == None:
        OPTPARSER.print_help()
        print '\nMissing arg --configfile'
        sys.exit(1)

    MakeUpdateinfo(OPTIONS.configfile, OPTIONS.verbose)
